function nn = nnbp(nn)
%NNBP performs backpropagation
% nn = nnbp(nn) returns an neural network structure with updated weights 
    
    n = nn.n;           % # of layers
    sparsityError = 0;  % sparsity error
    
    switch nn.output
        case 'sigm'               
            d{n} = - nn.e .* (nn.a{n} .* (1 - nn.a{n}));   
        case {'softmax','linear'}   
            d{n} = - nn.e;          
    end
    
    % Derivative of the activation function
    for i = (n - 1) : -1 : 2
        switch nn.activation_function 
            case 'sigm'             
                d_act = nn.a{i} .* (1 - nn.a{i});   
            case 'tanh_opt'       
                d_act = 1.7159 * 2/3 * (1 - 1/(1.7159)^2 * nn.a{i}.^2); 
            case 'tanh'           
                d_act = (1 - nn.a{i}.^2);             
        end
        
        % sparsity
        if(nn.nonSparsityPenalty>0)
            pi = repmat(nn.p{i}, size(nn.a{i}, 1), 1);  % pi : average hidden activation of vectors
            sparsityError = [zeros(size(nn.a{i},1),1) nn.nonSparsityPenalty * (-nn.sparsityTarget ./ pi + (1 - nn.sparsityTarget) ./ (1 - pi))]; % add bias (zeros)
        end
        
        if i+1==n       
            d{i} = (d{i + 1} * nn.W{i} + sparsityError) .* d_act; % Bishop (5.56)
        else 
            d{i} = (d{i + 1}(:,2:end) * nn.W{i} + sparsityError) .* d_act;
        end
        
        % dropout
        if(nn.dropoutFraction>0)
            d{i} = d{i} .* [ones(size(d{i},1),1) nn.dropOutMask{i}];
        end

    end

    for i = 1 : (n - 1)
        if i+1==n
            nn.dW{i} = (d{i + 1}' * nn.a{i}) / size(d{i + 1}, 1); 
        else
            nn.dW{i} = (d{i + 1}(:,2:end)' * nn.a{i}) / size(d{i + 1}, 1);      
        end
    end
    
end
